/**
 * Copyright 2006-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.log;

import java.io.Serializable;

import org.ow2.util.i18n.I18n;

/**
 * Interface used for the logging.<br>
 * This don't use directly commons logging as the aim of this logging is to
 * provide internationalization abilities and to allow to use MessageFormat
 * message with an array of objects.
 * @author Florent Benoit
 */

public interface Log extends Serializable {

    /**
     * <p>
     * Is debug logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than debug.
     * </p>
     * @return true if it is enabled, else false
     */
    boolean isDebugEnabled();

    /**
     * <p>
     * Is error logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than error.
     * </p>
     * @return true if it is enabled, else false
     */
    boolean isErrorEnabled();

    /**
     * <p>
     * Is fatal logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than fatal.
     * </p>
     * @return true if it is enabled, else false
     */
    boolean isFatalEnabled();

    /**
     * <p>
     * Is info logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than info.
     * </p>
     * @return true if it is enabled, else false
     */
    boolean isInfoEnabled();

    /**
     * <p>
     * Is trace logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than trace.
     * </p>
     * @return true if it is enabled, else false
     */
    boolean isTraceEnabled();

    /**
     * <p>
     * Is warn logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than warn.
     * </p>
     * @return true if it is enabled, else false
     */
    boolean isWarnEnabled();

    /**
     * <p>
     * Log a message with trace log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    void trace(final Object message, final Object... args);

    /**
     * <p>
     * Log a message with debug log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    void debug(final Object message, final Object... args);

    /**
     * <p>
     * Log a message with info log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    void info(final Object message, final Object... args);

    /**
     * <p>
     * Log a message with warn log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    void warn(final Object message, final Object... args);

    /**
     * <p>
     * Log a message with error log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    void error(final Object message, final Object... args);

    /**
     * <p>
     * Log a message with fatal log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    void fatal(final Object message, final Object... args);


    /**
     * Gets the i18n object associated to this logger.
     * @return i18n object.
     */
    I18n getI18n();

}
